'use strict';

var assert = require('assert');
var Validation = require('./Validation').Validation;

var _require = require('./Validators'),
    AsyncValue = _require.AsyncValue,
    EmailValue = _require.EmailValue;

var delayTime = 250;
var validationSchema = {
    email: {
        required: true,
        type: EmailValue
    },
    name: {
        required: {
            required: true,
            message: 'custom'
        }
    }
};

var data = {
    email: 'aaxbzsgher.developer@gmail.com',
    name: ''
};

var validation = new Validation(validationSchema);

describe('ValidationResults', function () {

    it('should contain fieldname of email and name respectively', function (done) {
        var obs$ = validation.validate(data);
        var counter = 0;
        obs$.subscribe(function (validationResult) {
            console.log(validationResult);
            assert.equal(['email', 'name'].includes(validationResult.fieldname), true);
            counter++;
            if (counter === 2) {
                done();
            }
        });
    });

    it('should contain message "custom"', function (done) {
        var obs$ = validation.validate({ name: data.name });
        obs$.subscribe(function (validationResult) {
            assert.equal(validationResult.errors[0], 'custom');
            done();
        });
    });
});